import type { ConvertibleToAsyncError } from "./";
import { AsyncError } from "./";
/**
 * MultiStateAsyncValue can have more than one
 * of its properties [value, pending, error] being truthy simultaneously
 */
export declare class MultiStateAsyncValue<T> {
    private static _loggedErrors;
    private static _consoleErrorLogger;
    static errorHanlder?: (error: Error) => void;
    static logErrorsToConsole(shouldLog?: boolean): void;
    private static _handleError;
    protected _error: Error;
    protected _pending: boolean;
    protected _value: T;
    constructor(value?: T, pending?: boolean, error?: ConvertibleToAsyncError);
    static errorOnly<T = any>(error: ConvertibleToAsyncError): MultiStateAsyncValue<T>;
    static pendingOnly<T = any>(): MultiStateAsyncValue<T>;
    static valueOnly<T>(value: T): MultiStateAsyncValue<T>;
    static createEmpty<T = any>(): MultiStateAsyncValue<T>;
    /**
     * Getters and Setters
     */
    get error(): AsyncError;
    set error(nextError: AsyncError);
    setErrorFrom(nextError: ConvertibleToAsyncError): void;
    get pending(): boolean;
    set pending(nextPending: boolean);
    get value(): T;
    set value(nextValue: T);
    /**
     * Clone methods
     */
    clone(): MultiStateAsyncValue<T>;
    cloneAndSetError(error: ConvertibleToAsyncError): MultiStateAsyncValue<T>;
    cloneWithNoError(): MultiStateAsyncValue<T>;
    cloneAndSetPending(nextPending?: boolean): MultiStateAsyncValue<T>;
    cloneWithNoPending(): MultiStateAsyncValue<T>;
    cloneAndSetValue(value: T): MultiStateAsyncValue<T>;
    cloneWithNoValue(): MultiStateAsyncValue<T>;
    /**
     * @returns true if not in pending state and there is no error
     * it does not care if value is null or not
     */
    get isFulfilled(): boolean;
    /**
     * @returns true if value is not equal to null | undefined
     */
    get hasValue(): boolean;
}
export declare type GenericAsyncValue<T> = MultiStateAsyncValue<T>;
