import { Observable } from 'rxjs';
import type { AVStreamEvent, StreamRenderingInfo } from "../..";
declare class AVWatchClass {
    /**
     * timestamp of the first watched event
     */
    private _initTimestamp;
    /**
     * all events from all the streams and all the phases
     */
    private _events;
    private _eventCount;
    /**
     * all the streams being watched
     */
    private _streams;
    private _streamCount;
    private _hiddenStreams;
    private _visibleStreams;
    constructor();
    get events(): Observable<AVStreamEvent[]>;
    get eventCount(): Observable<number>;
    addEvent(event: AVStreamEvent): void;
    addWatch(streamName: string, streamPhase: string): void;
    get streamCount(): Observable<number>;
    get initTimestamp(): number;
    setHiddenStreams(hiddenStreams: Set<string>): void;
    get visibleStreams(): Observable<StreamRenderingInfo[]>;
    latestEventByPhase(streamName: string, streamPhase: string): AVStreamEvent | null;
    /**
     *
     * searches for event of eventId in phase[streamPhase] of stream[streamName]
     * @param streamName name of the stream to search
     * @param streamPhase phase of the stream to search
     * @param eventId eventId to search for
     * @returns event of eventId if found or the newest event that happened before event[eventId]
     * @returns null if none exists
     */
    eventByIdOrOlder(streamName: string, streamPhase: string, eventId: number): AVStreamEvent | null;
    /**
     * searches for the event in the same phase as [event]
     * @param event - starting event
     * @param offset - i.e. if offset=-1, find previous event of same phase
     * @returns AVStreamEvent if found, null otherwise
     * @returns null if [event] is null
     */
    eventInSamePhase(event: AVStreamEvent | null, offset: number): AVStreamEvent | null;
    private _eventsView;
    private _eventsViewMountingSubscriber;
    activate({ showHideShortcut, showWatcherOnMount }: AVWatcherActivateOptions): void;
    deactivate(): void;
}
export interface AVWatcherActivateOptions {
    showHideShortcut?: string;
    showWatcherOnMount?: boolean;
}
export declare const AVWatch: AVWatchClass;
export * from './watch';
